"use strict";

exports.__esModule = true;
exports.createStyles = createStyles;
exports.compileStyles = compileStyles;

var _cloneDeep = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("clone-deep"));

var _cssToReactNative = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("css-to-react-native"));

var React = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react"));

var RN = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react-native"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const __reactCreateElement__ = React.createElement;
const emptyObj = {};

function createStyles(options = emptyObj) {
  var _options$themes, _options$tokens;

  const themes = (_options$themes = options.themes) !== null && _options$themes !== void 0 ? _options$themes : emptyObj;
  let currentTheme = "default";
  const tokens = {
    default: (_options$tokens = options.tokens) !== null && _options$tokens !== void 0 ? _options$tokens : emptyObj
  };

  if (options.themes) {
    const themeNames = Object.keys(options.themes);
    currentTheme = themeNames[0];

    for (const themeName of themeNames) {
      tokens[themeName] = mergeTokens((0, _cloneDeep.default)(tokens.default), themes[themeName]);
    }

    const colorScheme = RN.Appearance.getColorScheme();

    if (themeNames.length === 2 && "light" in tokens && "dark" in tokens && colorScheme && colorScheme in themes) {
      currentTheme = colorScheme;
    }
  }

  function cls(literals, ...placeholders) {
    const styles = compileStyles(compileLiterals(literals, ...placeholders), tokens[currentTheme]);

    if (process.env.NODE_ENV !== "production") {
      return Object.freeze(styles);
    }

    return styles;
  }

  const styles = Object.assign(function styles(styleMap) {
    // style('text', {})
    function style(...args) {
      const numArgs = args.length;
      const sheet = styleMap.default ? compileStyles(styleMap.default, tokens[currentTheme]) : {};

      if (numArgs === 1 && typeof args[0] === "string") {
        Object.assign(sheet, compileStyles(styleMap[args[0]], tokens[currentTheme]));
      } else if (numArgs > 0) {
        let i = 0;
        let arg;

        for (; i < numArgs; i++) {
          arg = args[i];

          if (typeof arg === "string") {
            Object.assign(sheet, compileStyles(styleMap[arg], tokens[currentTheme]));
          } else if (typeof arg === "object") {
            for (const key in arg) if (arg[key]) Object.assign(sheet, compileStyles(styleMap[key], tokens[currentTheme]));
          }
        }
      }

      return process.env.NODE_ENV !== "production" ? Object.freeze(sheet) : sheet;
    }

    style.styles = styleMap;
    return style;
  }, {
    one(literals, ...placeholders) {
      let styles = compileStyles(compileLiterals(literals, ...placeholders), tokens[currentTheme]);

      if (process.env.NODE_ENV !== "production") {
        styles = Object.freeze(styles);
      }

      return Object.assign(function oneStyle(createStyle) {
        if (!createStyle && createStyle !== void 0) return emptyObj;
        return styles;
      }, {
        styles
      });
    },

    cls,

    lazy(lazyFn) {
      return function (value) {
        if (!value) return emptyObj;
        let styles = compileStyles(lazyFn(value), tokens[currentTheme]);

        if (process.env.NODE_ENV !== "production") {
          styles = Object.freeze(styles);
        }

        return styles;
      };
    },

    join(...css) {
      return cls("".concat(...css));
    },

    tokens
  });
  const DashContext = /*#__PURE__*/React.createContext({
    styles,
    theme: currentTheme,

    setTheme(theme) {
      console.error("DashContext was consumed outside of a Provider");
    }

  });
  /**
   * A hook for consuming dash context from the provider
   */

  function useDash() {
    return React.useContext(DashContext);
  }

  const styled = function styled(Component, styles) {
    const RefForwardingComponent = /*#__PURE__*/React.forwardRef(function StyledComponent({
      style,
      ...props
    }, ref) {
      const {
        theme
      } = useDash();
      const baseStyle = compileStyles(styles, tokens[theme]);
      const outerStyle = typeof style === "function" || typeof style === "string" ? compileStyles(style, tokens[theme]) : style;
      return /*#__PURE__*/__reactCreateElement__(Component, { ...props,
        style: outerStyle && baseStyle ? RN.StyleSheet.flatten([baseStyle, outerStyle]) : outerStyle ? outerStyle : baseStyle,
        ref: ref
      });
    }); // if (process.env.NODE_ENV !== "production") {
    //   RefForwardingComponent.displayName = `styled(${
    //     Component.displayName ?? (Component.name || "Component")
    //   })`;
    // }

    return RefForwardingComponent;
  };

  function wrapStyled(Component) {
    return function styledWrapper(literals, ...placeholders) {
      return styled(Component, Array.isArray(literals) ? compileLiterals(literals, ...placeholders) : literals);
    };
  }

  styled.ActivityIndicator = wrapStyled(RN.ActivityIndicator);
  styled.DrawerLayoutAndroid = wrapStyled(RN.DrawerLayoutAndroid);
  styled.FlatList = wrapStyled(RN.FlatList);
  styled.Image = wrapStyled(RN.Image);
  styled.ImageBackground = wrapStyled(RN.ImageBackground);
  styled.KeyboardAvoidingView = wrapStyled(RN.KeyboardAvoidingView);
  styled.Modal = wrapStyled(RN.Modal);
  styled.NavigatorIOS = wrapStyled(RN.NavigatorIOS);
  styled.RecyclerViewBackedScrollView = wrapStyled(RN.RecyclerViewBackedScrollView);
  styled.RefreshControl = wrapStyled(RN.RefreshControl);
  styled.SafeAreaView = wrapStyled(RN.SafeAreaView);
  styled.ScrollView = wrapStyled(RN.ScrollView);
  styled.SectionList = wrapStyled(RN.SectionList);
  styled.SnapshotViewIOS = wrapStyled(RN.SnapshotViewIOS);
  styled.Switch = wrapStyled(RN.Switch);
  styled.Text = wrapStyled(RN.Text);
  styled.TextInput = wrapStyled(RN.TextInput);
  styled.TouchableHighlight = wrapStyled(RN.TouchableHighlight);
  styled.TouchableNativeFeedback = wrapStyled(RN.TouchableNativeFeedback);
  styled.TouchableOpacity = wrapStyled(RN.TouchableOpacity);
  styled.TouchableWithoutFeedback = wrapStyled(RN.TouchableWithoutFeedback);
  styled.View = wrapStyled(RN.View);
  return {
    styles,
    styled,
    useDash,

    DashProvider({
      defaultTheme,
      children
    }) {
      const colorScheme = RN.useColorScheme();
      const [theme, setTheme] = React.useState(() => {
        if (defaultTheme) {
          currentTheme = defaultTheme;
          return defaultTheme;
        }

        if (colorScheme && colorScheme in themes) {
          currentTheme = colorScheme;
          return colorScheme;
        }

        return currentTheme;
      });
      React.useLayoutEffect(() => {
        if (theme) currentTheme = theme;
      }, [theme]);
      React.useLayoutEffect(() => {
        if (colorScheme && colorScheme !== theme && // light + dark + default
        Object.values(themes).length === 3 && "light" in themes && "dark" in themes) {
          currentTheme = colorScheme;
        }
      }, [colorScheme, theme]);
      return /*#__PURE__*/__reactCreateElement__(DashContext.Provider, {
        value: React.useMemo(() => ({
          styles,
          theme,
          setTheme
        }), [theme]),
        children: children
      });
    }

  };
}
/**
 * A utility function that will compile style objects and callbacks into CSS strings.
 *
 * @param styles - A style callback, object, or string
 * @param tokens - A map of CSS tokens for style callbacks
 */


function compileStyles(styles, tokens) {
  const value = typeof styles === "function" ? styles(tokens) : styles;

  if (typeof value !== "string") {
    return value || {};
  }

  let cache = compileCache.get(tokens);

  if (!cache) {
    cache = new Map();
    compileCache.set(tokens, cache);
  }

  const cachedStyle = cache.get(value);
  if (cachedStyle) return cachedStyle;
  const stylePairs = [];
  const splitValue = value.split(";");

  for (let i = 0; i < splitValue.length; i++) {
    // Get prop name and prop value
    const match = propertyValuePattern.exec(splitValue[i]);
    if (match !== null) stylePairs.push([match[1], match[2]]);
  }

  try {
    const style = (0, _cssToReactNative.default)(stylePairs);
    cache.set(value, style);
    return style;
  } catch (error) {
    const msg = error.message;

    if (msg.includes("Failed to parse declaration")) {
      const values = msg.replace("Failed to parse declaration ", "").replace(/"/g, "").trim().split(":");
      console.error(`'${values[0]}' shorthand property requires units for example - ${values[0]}: 20px or ${values[0]}: 10px 20px 40px 50px`);
    }

    const style = {};
    cache.set(value, style);
    return style;
  }
}

const compileCache = /*#__PURE__*/new WeakMap();

function mergeTokens(target, source) {
  for (const key in source) {
    var _target$key;

    const value = source[key];
    target[key] = typeof value === "object" && value !== null && !Array.isArray(value) ? mergeTokens((_target$key = target[key]) !== null && _target$key !== void 0 ? _target$key : {}, value) : value;
  }

  return target;
}

const propertyValuePattern = /\s*([^\s]+)\s*:\s*(.+?)\s*$/;

function compileLiterals(literals, ...replacements) {
  return Array.isArray(literals) ? literals.reduce((curr, next, i) => curr + next + (replacements[i] || ""), "") : literals;
}